import pygame
from OpenGL.GL import *
from OpenGL.GLU import *
import math

class CVector3:
    def __init__( self, X=0 , Y=0 , Z=0 ):
	self.x = X
	self.y = Y
	self.z = Z
    def __add__( self,vVector):
	return CVector3(vVector.x + self.x, vVector.y + self.y, vVector.z + self.z)
    def __sub__( self,vVector):
	return CVector3(self.x - vVector.x, self.y - vVector.y, self.z - vVector.z)
    def __mult__( self,num):
	return CVector3(self.x * num, self.y * num, self.z * num)
    def __div__( self,num):
	return CVector3(self.x / num, self.y / num, self.z / num)
    def Mag(self):
	return math.sqrt(self.x*self.x+self.y*self.y+self.z*self.z)

def Normalize(vector):
    vector1=vector/vector.Mag()
    return vector1

def Cross(vect1, vect2):
    vect3=CVector3()
    vect3.x=((vect1.y * vect2.z) - (vect1.z * vect2.y))
    vect3.y=((vect1.z * vect2.x) - (vect1.x * vect2.z))
    vect3.z=((vect1.x * vect2.y) - (vect1.y * vect2.x))
    return vect3
   
kSpeed=1.5
class CCamera:
    def __init__(self):
    	self.m_vPosition = CVector3(0.0,50.0,200.0)
        self.m_vView     = CVector3(0.0,0.0,0.0)
	self.m_vUpVector = CVector3(0.0,1.0,0.0)
        self.m_vStrafe   = CVector3()
	self.upPressed   = False
	self.downPressed = False
	self.leftPressed = False
	self.rightPressed= False
    
    def Position( self):
	return self.m_vPosition
    
    def View( self):
	return self.m_vView
    
    def UpVector( self):
	return self.m_vUpVector
    
    def Strafe( self):
	return self.m_vStrafe
    
    def PositionCamera( self,positionX,positionY,positionZ,viewX,viewY,viewZ,upVectorX,upVectorY,upVectorZ):
	self.m_vPosition =CVector3(positionX, positionY, positionZ);
        self.m_vView     =CVector3(viewX, viewY, viewZ);
	self.m_vUpVector =CVector3(upVectorX, upVectorY, upVectorZ);

    def RotateView( self,angle,x, y, z):
	vNewView = CVector3()
        vView    = self.m_vView - self.m_vPosition;        

	cosTheta = math.cos(angle)
	sinTheta = math.sin(angle)

	vNewView.x  = (cosTheta + (1 - cosTheta) * x * x)        * vView.x
        vNewView.x += ((1 - cosTheta) * x * y - z * sinTheta)    * vView.y
	vNewView.x += ((1 - cosTheta) * x * z + y * sinTheta)    * vView.z

	vNewView.y  = ((1 - cosTheta) * x * y + z * sinTheta)    * vView.x
        vNewView.y += (cosTheta + (1 - cosTheta) * y * y)        * vView.y
	vNewView.y += ((1 - cosTheta) * y * z - x * sinTheta)    * vView.z

	vNewView.z  = ((1 - cosTheta) * x * z - y * sinTheta)    * vView.x
        vNewView.z += ((1 - cosTheta) * y * z + x * sinTheta)    * vView.y
	vNewView.z += (cosTheta + (1 - cosTheta) * z * z)        * vView.z

	self.m_vView = self.m_vPosition + vNewView

    def SetViewByMouse( self):
	middleX = 640/2
	middleY = 480/2
	angleY = 0.0
	angleZ = 0.0
	currentRotX = 0.0
#	mouseX,mouseY=SDL_GetMouseState()
	mouseX, mouseY = pygame.mouse.get_pos( );
	if( (mouseX == middleX) and (mouseY == middleY) ):
	    return
	#SDL_WarpMouse(middleX, middleY)
	pygame.mouse.set_pos(middleX, middleY);
	angleY = (float)( (middleX - mouseX) ) / 1000.0
	angleZ = (float)( (middleY - mouseY) ) / 1000.0
	currentRotX -= angleZ
	if(currentRotX > 1.0):
	    currentRotX = 1.0
	elif(currentRotX < -1.0):
	    currentRotX = -1.0
	else:
	    vAxis = Cross(self.m_vView - self.m_vPosition, self.m_vUpVector)
	    vAxis = Normalize(vAxis)
	    self.RotateView(angleZ, vAxis.x, vAxis.y, vAxis.z)
	    self.RotateView(angleY, 0, 1, 0)

    def RotateAroundPoint( self,vCenter,X,Y,Z):
	pass
    
    def StrafeCamera( self,speed):
	self.m_vPosition.x += self.m_vStrafe.x * speed
        self.m_vPosition.z += self.m_vStrafe.z * speed
	self.m_vView.x += self.m_vStrafe.x * speed
        self.m_vView.z += self.m_vStrafe.z * speed
    
    def MoveCamera( self,speed):
	vVector = self.m_vView - self.m_vPosition
	vVector = Normalize(vVector)
	self.m_vPosition.x += vVector.x * speed
	self.m_vPosition.z += vVector.z * speed
	self.m_vView.x += vVector.x * speed
	self.m_vView.z += vVector.z * speed
    
    def CheckForMovement( self):
	oldpos= CVector3(self.m_vPosition.x,self.m_vPosition.y,self.m_vPosition.z);
	if(self.upPressed):
            self.MoveCamera(kSpeed)
	if(self.downPressed):
            self.MoveCamera(-kSpeed)
	if(self.leftPressed):
            self.StrafeCamera(-kSpeed)
	if(self.rightPressed):
            self.StrafeCamera(kSpeed)
	d= math.sqrt(self.m_vPosition.x*self.m_vPosition.x+self.m_vPosition.z*self.m_vPosition.z) 
	if d > 1000 or d< 100 :
	    self.m_vPosition=oldpos;
    
    def Update( self):
        vCross = Cross(self.m_vView - self.m_vPosition, self.m_vUpVector)
        self.m_vStrafe = Normalize(vCross)
        self.SetViewByMouse()
        self.CheckForMovement()
    def Look( self):
        gluLookAt(self.m_vPosition.x, self.m_vPosition.y, self.m_vPosition.z,self.m_vView.x,self.m_vView.y,self.m_vView.z,self.m_vUpVector.x, self.m_vUpVector.y, self.m_vUpVector.z)
